@@ -1,3 +1,4 @@
+/************************** BEGIN juce-midi.h **************************/
 /************************************************************************
  FAUST Architecture File
  Copyright (C) 2003-2017 GRAME, Centre National de Creation Musicale
@@ -28,70 +29,46 @@
 
 class MapUI;
 
-//-----------------------------------------------
-// MIDI input/output handling using JUCE library
-//-----------------------------------------------
-
+/**
+ *  MIDI input/output handling using JUCE framework: https://juce.com
+ */
 class juce_midi_handler : public midi_handler {
     
     protected:
     
-        MidiBuffer fOutputBuffer;
-        CriticalSection fMutex;
+        juce::MidiBuffer fOutputBuffer;
+        juce::CriticalSection fMutex;
     
-        void decodeMessage(const MidiMessage& message)
+        void decodeMessage(const juce::MidiMessage& message)
         {
-            const uint8* data = message.getRawData();
+            const juce::uint8* data = message.getRawData();
+            int channel = message.getChannel() - 1; // which MIDI channel, 0-15
+            double time = message.getTimeStamp();
             
             if (message.isNoteOff()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->keyOff(message.getTimeStamp(), message.getChannel(), data[1], data[2]);
-                }
+                handleKeyOff(time, channel, data[1], data[2]);
             } else if (message.isNoteOn()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    if (data[1] != 0) {
-                        fMidiInputs[i]->keyOn(message.getTimeStamp(), message.getChannel(), data[1], data[2]);
-                    } else {
-                        fMidiInputs[i]->keyOff(message.getTimeStamp(), message.getChannel(), data[1], data[2]);
-                    }
-                }
+                handleKeyOn(time, channel, data[1], data[2]);
             } else if (message.isAftertouch()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->keyPress(message.getTimeStamp(), message.getChannel(), data[1], data[2]);
-                }
+                handlePolyAfterTouch(time, channel, data[1], data[2]);
             } else if (message.isController()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->ctrlChange(message.getTimeStamp(), message.getChannel(), data[1], data[2]);
-                }
+                handleCtrlChange(time, channel, data[1], data[2]);
             } else if (message.isProgramChange()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->progChange(message.getTimeStamp(), message.getChannel(), data[1]);
-                }
+                handleProgChange(time, channel, data[1]);
             } else if (message.isChannelPressure()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->chanPress(message.getTimeStamp(), message.getChannel(), data[1]);
-                }
+                handleAfterTouch(time, channel, data[1]);
             } else if (message.isPitchWheel()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->pitchWheel(message.getTimeStamp(), message.getChannel(), ((data[1] << 7) + data[2]));
-                }
+                handlePitchWheel(time, channel, data[1], data[2]);
             } else if (message.isMidiClock()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->clock(message.getTimeStamp());
-                }
-            } else if (message.isMidiStart()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->startSync(message.getTimeStamp());
-                }
+                handleClock(time);
+            // We can consider start and continue as identical messages.
+            } else if (message.isMidiStart() || message.isMidiContinue()) {
+                handleStart(time);
             } else if (message.isMidiStop()) {
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->stopSync(message.getTimeStamp());
-                }
+                handleStop(time);
             } else if (message.isSysEx()) {
                 std::vector<unsigned char> sysex(data, data + message.getRawDataSize());
-                for (unsigned int i = 0; i < fMidiInputs.size(); i++) {
-                    fMidiInputs[i]->sysEx(message.getTimeStamp(), sysex);
-                }
+                handleSysex(time, sysex);
             } else {
                 std::cerr << "Unused MIDI message" << std::endl;
             }
@@ -105,9 +82,9 @@ class juce_midi_handler : public midi_handler {
         virtual ~juce_midi_handler() {}
     
         // Used with MidiBuffer (containing several messages)
-        void encodeBuffer(MidiBuffer& buffer)
+        void encodeBuffer(juce::MidiBuffer& buffer)
         {
-            const ScopedTryLock lock(fMutex);
+            const juce::ScopedTryLock lock(fMutex);
             if (lock.isLocked()) {
                 buffer.swapWith(fOutputBuffer);
                 fOutputBuffer.clear();
@@ -116,53 +93,51 @@ class juce_midi_handler : public midi_handler {
             }
         }
         
-        void decodeBuffer(MidiBuffer& buffer)
+        void decodeBuffer(juce::MidiBuffer& buffer)
         {
-            MidiMessage msg;
+            juce::MidiMessage msg;
             int ignore;
-
-            for (MidiBuffer::Iterator it(buffer); it.getNextEvent(msg, ignore);) {
+            for (juce::MidiBuffer::Iterator it(buffer); it.getNextEvent(msg, ignore);) {
                 decodeMessage(msg);
             }
-            
             buffer.clear();
         }
     
         // MIDI output API
         MapUI* keyOn(int channel, int pitch, int velocity)
         {
-            fOutputBuffer.addEvent(MidiMessage::noteOn(channel + 1, pitch, uint8(velocity)), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::noteOn(channel + 1, pitch, juce::uint8(velocity)), 0);
             return nullptr;
         }
         
         void keyOff(int channel, int pitch, int velocity)
         {
-            fOutputBuffer.addEvent(MidiMessage::noteOff(channel + 1, pitch, uint8(velocity)), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::noteOff(channel + 1, pitch, juce::uint8(velocity)), 0);
         }
         
         void ctrlChange(int channel, int ctrl, int val)
         {
-            fOutputBuffer.addEvent(MidiMessage::controllerEvent(channel + 1, ctrl, uint8(val)), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::controllerEvent(channel + 1, ctrl, juce::uint8(val)), 0);
         }
         
         void chanPress(int channel, int press)
         {
-            fOutputBuffer.addEvent(MidiMessage::channelPressureChange(channel + 1, press), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::channelPressureChange(channel + 1, press), 0);
         }
         
         void progChange(int channel, int pgm)
         {
-            fOutputBuffer.addEvent(MidiMessage::programChange(channel + 1, pgm), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::programChange(channel + 1, pgm), 0);
         }
         
         void keyPress(int channel, int pitch, int press)
         {
-            fOutputBuffer.addEvent(MidiMessage::aftertouchChange(channel + 1, pitch, press), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::aftertouchChange(channel + 1, pitch, press), 0);
         }
         
         void pitchWheel(int channel, int wheel)
         {
-            fOutputBuffer.addEvent(MidiMessage::pitchWheel(channel + 1, range(0, 16383, wheel)), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::pitchWheel(channel + 1, range(0, 16383, wheel)), 0);
         }
         
         void ctrlChange14bits(int channel, int ctrl, int value)
@@ -172,40 +147,43 @@ class juce_midi_handler : public midi_handler {
         
         void startSync(double date)
         {
-            fOutputBuffer.addEvent(MidiMessage::midiStart(), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::midiStart(), 0);
         }
         
         void stopSync(double date)
         {
-            fOutputBuffer.addEvent(MidiMessage::midiStop(), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::midiStop(), 0);
         }
         
         void clock(double date)
         {
-            fOutputBuffer.addEvent(MidiMessage::midiClock(), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage::midiClock(), 0);
         }
     
         void sysEx(double date, std::vector<unsigned char>& message)
         {
-            fOutputBuffer.addEvent(MidiMessage(message.data(), (int)message.size()), 0);
+            fOutputBuffer.addEvent(juce::MidiMessage(message.data(), (int)message.size()), 0);
         }
 
 };
 
-class juce_midi : public juce_midi_handler, public MidiInputCallback {
+class juce_midi : public juce_midi_handler, public juce::MidiInputCallback {
 
     private:
     
-        MidiInput* fMidiIn;
-        MidiOutput* fMidiOut;
+        std::unique_ptr<juce::MidiInput> fMidiIn;
+        std::unique_ptr<juce::MidiOutput> fMidiOut;
     
-        void handleIncomingMidiMessage(MidiInput*, const MidiMessage& message)
+        void handleIncomingMidiMessage(juce::MidiInput*, const juce::MidiMessage& message)
         {
             decodeMessage(message);
         }
     
     public:
     
+        juce_midi():juce_midi_handler()
+        {}
+    
         virtual ~juce_midi()
         {
             stopMidi();
@@ -213,13 +191,12 @@ class juce_midi : public juce_midi_handler, public MidiInputCallback {
         
         bool startMidi()
         {
-            if ((fMidiIn = MidiInput::openDevice(MidiInput::getDefaultDeviceIndex(), this)) == nullptr) {
+            if ((fMidiIn = juce::MidiInput::openDevice(juce::MidiInput::getDefaultDeviceIndex(), this)) == nullptr) {
                 return false;
             }
-            if ((fMidiOut = MidiOutput::openDevice(MidiOutput::getDefaultDeviceIndex())) == nullptr) {
+            if ((fMidiOut = juce::MidiOutput::openDevice(juce::MidiOutput::getDefaultDeviceIndex())) == nullptr) {
                 return false;
             }
-            
             fMidiIn->start();
             return true;
         }
@@ -227,45 +204,43 @@ class juce_midi : public juce_midi_handler, public MidiInputCallback {
         void stopMidi()
         { 
             fMidiIn->stop();
-            delete fMidiIn;
-            delete fMidiOut;
         }
     
         // MIDI output API
         MapUI* keyOn(int channel, int pitch, int velocity)
         {
-            fMidiOut->sendMessageNow(MidiMessage::noteOn(channel + 1, pitch, uint8(velocity)));
+            fMidiOut->sendMessageNow(juce::MidiMessage::noteOn(channel + 1, pitch, juce::uint8(velocity)));
             return nullptr;
         }
         
         void keyOff(int channel, int pitch, int velocity) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::noteOff(channel + 1, pitch, uint8(velocity)));
+            fMidiOut->sendMessageNow(juce::MidiMessage::noteOff(channel + 1, pitch, juce::uint8(velocity)));
         }
         
         void ctrlChange(int channel, int ctrl, int val) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::controllerEvent(channel + 1, ctrl, uint8(val)));
+            fMidiOut->sendMessageNow(juce::MidiMessage::controllerEvent(channel + 1, ctrl, juce::uint8(val)));
         }
         
         void chanPress(int channel, int press) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::channelPressureChange(channel + 1, press));
+            fMidiOut->sendMessageNow(juce::MidiMessage::channelPressureChange(channel + 1, press));
         }
         
         void progChange(int channel, int pgm) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::programChange(channel + 1, pgm));
+            fMidiOut->sendMessageNow(juce::MidiMessage::programChange(channel + 1, pgm));
         }
           
         void keyPress(int channel, int pitch, int press) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::aftertouchChange(channel + 1, pitch, press));
+            fMidiOut->sendMessageNow(juce::MidiMessage::aftertouchChange(channel + 1, pitch, press));
         }
    
         void pitchWheel(int channel, int wheel) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::pitchWheel(channel + 1, range(0, 16383, wheel)));
+            fMidiOut->sendMessageNow(juce::MidiMessage::pitchWheel(channel + 1, range(0, 16383, wheel)));
         }
         
         void ctrlChange14bits(int channel, int ctrl, int value)
@@ -275,25 +250,26 @@ class juce_midi : public juce_midi_handler, public MidiInputCallback {
     
         void startSync(double date) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::midiStart());
+            fMidiOut->sendMessageNow(juce::MidiMessage::midiStart());
         }
        
         void stopSync(double date) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::midiStop());
+            fMidiOut->sendMessageNow(juce::MidiMessage::midiStop());
         }
         
         void clock(double date) 
         {
-            fMidiOut->sendMessageNow(MidiMessage::midiClock());
+            fMidiOut->sendMessageNow(juce::MidiMessage::midiClock());
         }
     
         void sysEx(double date, std::vector<unsigned char>& message)
         {
-            fMidiOut->sendMessageNow(MidiMessage(message.data(), (int)message.size()));
+            fMidiOut->sendMessageNow(juce::MidiMessage(message.data(), (int)message.size()));
         }
     
 };
 
 #endif // __juce_midi__
 
+/**************************  END  juce-midi.h **************************/
